package net.wanji.utc.service.control.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.po.BaseCrossLightsPO;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.SaveToUtcPO;
import net.wanji.databus.vo.*;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.cache.CrossSchemePhaseTimeCountCache;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.common.typeenum.DateStyle;
import net.wanji.utc.po.hk.request.DelBaseConfigPO;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.service.control.ControlCommandStrategyService;
import net.wanji.utc.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/12 9:53
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class ControlCommandStrategyServiceImpl implements ControlCommandStrategyService {

    @Resource(name = "hkControlCommandService")
    ControlCommandService hkControlCommandService;
    @Resource(name = "wanJiControlCommandService")
    ControlCommandService wanJiControlCommandService;
    private final CrossInfoMapper crossInfoMapper;
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossInfoCache crossInfoCache;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPhaseDirTurnCache crossPhaseDirTurnCache;
    private final CrossSchemePhaseTimeCountCache crossSchemePhaseTimeCountCache;


    @Override
    public JsonViewObject schemeSendStrategy(SchemeSendVO schemeSendVO) throws Exception {
        schemeSendDataInsert(schemeSendVO);
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(schemeSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.schemeSend(schemeSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.schemeSend(schemeSendVO);
        }
        return jsonViewObject;
    }

    @Transactional(rollbackFor = Exception.class)
    public void schemeSendDataInsert(SchemeSendVO schemeSendVO) throws Exception {
        try {
            // 入库操作
            List<CrossSchemePO> crossSchemePOList = new ArrayList<>();
            schemeSendVO.getPatternList().forEach(pattern -> {
                CrossSchemePO crossSchemePO = new CrossSchemePO();
                crossSchemePO.setSchemeNo(pattern.getPatternNo());
                crossSchemePO.setName(pattern.getPatternName());
                crossSchemePO.setCrossId(schemeSendVO.getCrossCode());
                crossSchemePO.setCycle(Integer.parseInt(pattern.getCycle()));
                crossSchemePO.setCoordPhase(Integer.parseInt(pattern.getCoordPhase()));
                crossSchemePO.setOffset(Integer.parseInt(pattern.getOffset()));
                crossSchemePO.setSource(2);
                crossSchemePO.setIsDeleted(0);
                crossSchemePOList.add(crossSchemePO);

                pattern.getRings().forEach(ring -> {
                    List<CrossPhasePO> crossPhasePOList = new ArrayList<>();
                    ring.getPhaseList().forEach(phase -> {
                        CrossPhasePO crossPhasePO = new CrossPhasePO();
                        crossPhasePO.setPhaseNo(phase.getPhaseNo());
                        crossPhasePO.setName(phase.getPhaseName());
                        crossPhasePO.setSort(Integer.parseInt(phase.getSort()));
                        crossPhasePO.setCrossId(schemeSendVO.getCrossCode());
                        crossPhasePO.setPlanId(Integer.parseInt(pattern.getPatternNo()));
                        crossPhasePO.setRingNo(Integer.parseInt(ring.getRingNo()));
                        crossPhasePO.setControlMode(Integer.parseInt(phase.getControlMode()));
                        crossPhasePO.setPhaseTime(Integer.parseInt(phase.getPhaseTime()));
                        crossPhasePO.setGreenTime(Integer.parseInt(phase.getGreenTime()));
                        crossPhasePO.setGreenFlashTime(Integer.parseInt(phase.getGreenFlashTime()));
                        crossPhasePO.setPedFlashTime(Integer.parseInt(phase.getPedFlashTime()));
                        crossPhasePO.setYellowTime(Integer.parseInt(phase.getYellowTime()));
                        crossPhasePO.setRedTime(Integer.parseInt(phase.getRedTime()));
                        crossPhasePO.setMinGreenTime(Integer.parseInt(phase.getMinGreenTime()));
                        crossPhasePO.setMaxGreenTime(Integer.parseInt(phase.getMaxGreenTime()));
                        crossPhasePOList.add(crossPhasePO);
                    });
                    // 入库路口相位表
                    crossPhaseMapper.insertBatch(crossPhasePOList);
                });

            });
            // 入库信号方案表
            crossSchemeMapper.insertBatch(crossSchemePOList);
        } catch (Exception e) {
            log.error("下发方案相位数据到utc数据库失败", e);
            throw new Exception("下发方案相位数据到utc数据库失败");
        }
    }

    @Override
    public JsonViewObject planSendStrategy(PlanSendVO planSendVO) throws Exception {
        planSendDataInsert(planSendVO);
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(planSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.planSend(planSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.planSend(planSendVO);
        }
        return jsonViewObject;
    }

    @Transactional(rollbackFor = Exception.class)
    public void planSendDataInsert(PlanSendVO planSendVO) throws Exception {
        try {
            List<CrossPlanPO> crossPlanPOList = new ArrayList<>();
            planSendVO.getPlanList().forEach(plan -> {
                CrossPlanPO crossPlanPO = new CrossPlanPO();
                crossPlanPO.setPlanNo(plan.getPlanNo());
                crossPlanPO.setName(plan.getPlanDescribe());
                crossPlanPO.setCrossId(planSendVO.getCrossCode());
                crossPlanPOList.add(crossPlanPO);
                List<CrossSectionPO> crossSectionPOList = new ArrayList<>();
                plan.getSectionList().forEach(section -> {
                    CrossSectionPO crossSectionPO = new CrossSectionPO();
                    crossSectionPO.setSectionNo(section.getSectionNo());
                    crossSectionPO.setStartTime(section.getBeginTime());
                    crossSectionPO.setEndTime(section.getEndTime());
                    crossSectionPO.setCrossId(planSendVO.getCrossCode());
                    crossSectionPO.setPlanId(Integer.parseInt(plan.getPlanNo()));
                    crossSectionPO.setControlMode(Integer.parseInt(section.getControlMode()));
                    crossSectionPO.setSchemeId(Integer.parseInt(section.getPatternNo()));
                    crossSectionPOList.add(crossSectionPO);
                });
                // 入库路口时段表
                crossSectionMapper.insertBatch(crossSectionPOList);
            });
            // 入库路口计划表
            crossPlanMapper.insertBatch(crossPlanPOList);
        } catch (Exception e) {
            log.error("下发时段日计划数据到UTC数据库失败", e);
            throw new Exception("下发时段日计划数据到UTC数据库失败");
        }
    }

    @Override
    public JsonViewObject scheduleSendStrategy(ScheduleSendVO scheduleSendVO) throws Exception {
        scheduleSendDataInsert(scheduleSendVO);
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(scheduleSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.scheduleSend(scheduleSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.scheduleSend(scheduleSendVO);
        }
        return jsonViewObject;
    }

    @Transactional(rollbackFor = Exception.class)
    public void scheduleSendDataInsert(ScheduleSendVO scheduleSendVO) throws Exception {
        try {
            // 1、入库操作
            SimpleDateFormat sdf = new SimpleDateFormat(DateStyle.YYYY_MM_DD.getValue());
            ArrayList<CrossSchedulesPO> crossSchedulesPOS = new ArrayList<>();
            scheduleSendVO.getSchedules().forEach(schedule -> {
                schedule.getWeeks().forEach(week -> {
                    CrossSchedulesPO schedulesPO = new CrossSchedulesPO();
                    if (week.getWeekNum() >= 6) {
                        schedulesPO.setName("周末");
                    } else {
                        schedulesPO.setName("工作日");
                    }
                    schedulesPO.setCrossId(scheduleSendVO.getCrossCode());
                    schedulesPO.setScheduleNo(Integer.valueOf(schedule.getScheduleNo()));
                    schedulesPO.setPlanId(week.getPlanNo());
                    schedulesPO.setWeek(week.getWeekNum());
                    schedulesPO.setSpecialDate(null);
                    crossSchedulesPOS.add(schedulesPO);
                });
                schedule.getSpecialDays().forEach(specialDay -> {
                    CrossSchedulesPO schedulesPO = new CrossSchedulesPO();
                    schedulesPO.setScheduleNo(Integer.valueOf(schedule.getScheduleNo()));
                    schedulesPO.setName("特殊日期");
                    schedulesPO.setCrossId(scheduleSendVO.getCrossCode());
                    schedulesPO.setPlanId(specialDay.getPlanNo());
                    schedulesPO.setWeek(null);
                    try {
                        schedulesPO.setSpecialDate(sdf.parse(specialDay.getDateStr()));
                    } catch (ParseException e) {
                        schedulesPO.setSpecialDate(null);
                    }
                    crossSchedulesPOS.add(schedulesPO);
                });
            });
            // 入库时间表
            crossSchedulesMapper.insertBatch(crossSchedulesPOS);
        } catch (Exception e) {
            log.error("下发调度数据到utc数据库失败", e);
            throw new Exception("下发调度数据到utc数据库失败");
        }
    }

    @Override
    public JsonViewObject lockControlStrategy(ControlCommandVO commandVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.lockControl(commandVO);
        } else {
            jsonViewObject = wanJiControlCommandService.lockControl(commandVO);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepControlStrategy(String crossId, Integer command, Integer stepNum) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.stepControl(crossId, command, stepNum);
        } else {
            jsonViewObject = wanJiControlCommandService.stepControl(crossId, command, stepNum);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject setSignalControlStrategy(String crossId, Integer command, Integer commandType) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.setSignalControl(crossId, command, commandType);
        } else {
            jsonViewObject = wanJiControlCommandService.setSignalControl(crossId, command, commandType);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject recoverScheduleStrategy(String crossId) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.isBlank(crossId)) {
            return jsonViewObject.fail("路口编号不能为空");
        }
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.recoverSchedule(crossId);
        } else {
            jsonViewObject = wanJiControlCommandService.recoverSchedule(crossId);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseTimingSendStrategy(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        JsonViewObject jsonViewObject;
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(phaseTimingSendVO.getCrossCode());
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
            jsonViewObject = hkControlCommandService.phaseTimingSend(phaseTimingSendVO);
        } else {
            jsonViewObject = wanJiControlCommandService.phaseTimingSend(phaseTimingSendVO);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseDiffSendStrategy() {
        return null;
    }

    @Override
    public JsonViewObject delBaseConfig(DelBaseConfigVO delBaseConfigVO) throws Exception {
        // 通过路口编号获取信号机编号
        String crossId = delBaseConfigVO.getCrossId();
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = null;
        if (Objects.nonNull(crossInfoPO)) {
            crossCode = crossInfoPO.getCode();
        }
        DelBaseConfigPO delBaseConfigPO = new DelBaseConfigPO();
        delBaseConfigPO.setCrossCode(crossCode);
        delBaseConfigPO.setType(delBaseConfigVO.getType());
        delBaseConfigPO.setIds(delBaseConfigVO.getIds());
        // 通过厂商编码调用删除服务
        String manufacturer = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject;
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturer)) {
            jsonViewObject = hkControlCommandService.delBaseConfig(delBaseConfigPO);
        } else {
            jsonViewObject = wanJiControlCommandService.delBaseConfig(delBaseConfigPO);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject timing(String crossCode, Date date) throws Exception {
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(crossCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            jsonViewObject = hkControlCommandService.timing(crossCode, date);
        } else {
            jsonViewObject = wanJiControlCommandService.timing(crossCode, date);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject tempScheme(Object obj) throws Exception {
        String crossCode = getCrossCode(obj);
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(crossCode);
        JsonViewObject jsonViewObject = wanJiControlCommandService.tempScheme(obj);
        return jsonViewObject;
}

    public static String getCrossCode(Object obj) throws NoSuchFieldException, IllegalAccessException {
        // 获取对象的Class对象
        Class<?> clazz = obj.getClass();

        // 查找名为"crossCode"的字段（假设字段名是crossCode，且是public或可访问的）
        Field crossCodeField = clazz.getDeclaredField("crossCode");

        // 如果字段是私有的，则需要设置可访问性
        crossCodeField.setAccessible(true);

        // 获取字段的值
        Object crossCodeValue = crossCodeField.get(obj);

        // 将值转换为String类型（假设crossCode是String类型）
        return (String) crossCodeValue;
    }
}

    @Override
    public JsonViewObject tempSchemeDetail(TempSchemeSendVO tempSchemeSendVO) throws Exception {
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(tempSchemeSendVO.getCrossCode());
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
        } else {
            jsonViewObject = wanJiControlCommandService.tempSchemeDetail(tempSchemeSendVO);
        }
        return jsonViewObject;
    }

    @Override
    @Transactional
    public JsonViewObject saveToUtc(SaveToUtcPO saveToUtcPO) throws Exception {
        try {
            if (Objects.nonNull(saveToUtcPO)) {
                List<BaseCrossLightsPO> lightsPOS = saveToUtcPO.getLightsPOS();
                CrossSchedulesPO crossSchedulesPO = saveToUtcPO.getSchedulesPOS().get(0);
                String crossId = crossSchedulesPO.getCrossId();
                if (!CollectionUtils.isEmpty(lightsPOS)) {
                    crossLightsMapper.deleteByCrossId(crossId);
                    for (BaseCrossLightsPO lightsPO : lightsPOS) {
                        CrossLightsPO crossLightsPO = new CrossLightsPO();
                        BeanUtils.copyProperties(lightsPO, crossLightsPO);
                        crossLightsPO.setName("灯组".concat(lightsPO.getLightsNo()));
                        crossLightsPO.setType(1);
                        if (Objects.equals(lightsPO.getType(), 1)) {
                            crossLightsPO.setType(2); // 1箭头 2圆饼
                            crossLightsPO.setTurn(2);
                        } else if (Objects.equals(lightsPO.getType(), 2)) {
                            crossLightsPO.setTurn(1);
                        } else if (Objects.equals(lightsPO.getType(), 3)) {
                            crossLightsPO.setTurn(2);
                        } else if (Objects.equals(lightsPO.getType(), 4)) {
                            crossLightsPO.setTurn(3);
                        } else if (Objects.equals(lightsPO.getType(), 20)) {
                            crossLightsPO.setType(3);
                            crossLightsPO.setTurn(20);
                        }
                        crossLightsMapper.insertOne(crossLightsPO);
                    }
                }

                List<CrossPhasePO> phasePOS = saveToUtcPO.getPhasePOS();
                crossPhaseMapper.deleteByCrossId(crossId);
                if (!CollectionUtils.isEmpty(phasePOS)) {
                    for (CrossPhasePO phasePO : phasePOS) {
                        CrossPhasePO crossPhasePO = new CrossPhasePO();
                        BeanUtils.copyProperties(phasePO, crossPhasePO);
                        crossPhasePO.setName("相位".concat(phasePO.getPhaseNo()));
                        crossPhasePO.setGroupNo(1);
                        crossPhasePO.setRingNo(1);
                        crossPhaseMapper.insertOne(crossPhasePO);
                    }
                }
                List<CrossPhaseLightsPO> phaseLightsPOS = saveToUtcPO.getPhaseLightsPOS();
                crossPhaseLightsMapper.deleteByCrossId(crossId);
                crossPhaseLightsMapper.insertBatch(phaseLightsPOS);

                crossSchemeMapper.deleteByCrossId(crossId);
                List<BaseCrossSchemePO> schemePOS = saveToUtcPO.getSchemePOS();
                if (!CollectionUtils.isEmpty(schemePOS)) {
                    for (BaseCrossSchemePO schemePO : schemePOS) {
                        CrossSchemePO crossSchemePO = new CrossSchemePO();
                        BeanUtils.copyProperties(schemePO, crossSchemePO);
                        crossSchemeMapper.insertOne(crossSchemePO);
                    }
                }

                List<CrossSectionPO> sectionPOS = saveToUtcPO.getSectionPOS();
                crossSectionMapper.deleteOne(crossId, null, null);
                if (!CollectionUtils.isEmpty(sectionPOS)) {
                    for (CrossSectionPO sectionPO : sectionPOS) {
                        CrossSectionPO crossSectionPO = new CrossSectionPO();
                        BeanUtils.copyProperties(sectionPO, crossSectionPO);
                        crossSectionMapper.insertOne(crossSectionPO);
                    }
                }

                List<BaseCrossPlanPO> planPOS = saveToUtcPO.getPlanPOS();
                if (!CollectionUtils.isEmpty(planPOS)) {
                    for (BaseCrossPlanPO planPO : planPOS) {
                        CrossPlanPO crossPlanPO = new CrossPlanPO();
                        BeanUtils.copyProperties(planPO, crossPlanPO);
                        crossPlanMapper.deleteOne(planPO.getCrossId(), Integer.valueOf(crossPlanPO.getPlanNo()));
                        crossPlanMapper.insertOne(crossPlanPO);
                    }
                }
                List<CrossSchedulesPO> schedulesPOS = saveToUtcPO.getSchedulesPOS();
                crossSchedulesMapper.deleteByCrossId(crossId);
                crossSchedulesMapper.insertBatch(schedulesPOS);
            }
        } catch (Exception e) {
            log.error("下发数据到utc数据库失败：", e);
            throw new Exception(e);
        }
        crossPhaseDirTurnCache.run();
        crossSchemePhaseTimeCountCache.run();
        return JsonViewObject.newInstance().success("数据下发到Utc数据库成功，并刷新缓存");
    }

    @Override
    public JsonViewObject extendPhase(ExtendPhaseDTO extendPhase) throws Exception {
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(extendPhase.getCrossId());
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
        } else {
            jsonViewObject = wanJiControlCommandService.extendPhase(extendPhase);
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepingPhase(StepingPhaseDTO stepingPhaseDTO) throws Exception {
        String manufacturerCode = crossInfoCache.getManufacturerCodeByCrossId(stepingPhaseDTO.getCrossId());
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
        } else {
            jsonViewObject = wanJiControlCommandService.stepingPhase(stepingPhaseDTO);
        }
        return jsonViewObject;
    }
}
